unit Test.UClass.TAritmeticaBasica;


//==============================================================================
//
//  I N T E R F A C E
//
//==============================================================================
interface

uses
  {DUnit} // TestFramework,
  {DUnitX} DUnitX.TestFramework,DUnitX.DUnitCompatibility,
  System.Generics.Collections, UClass.TAritmeticaBasica;

type
  // Test methods for class TAritmeticaBasica
  TestTAritmeticaBasica = class(TTestCase)
  strict private
    FAritmeticaBasica: TAritmeticaBasica;
    FFactorizacion:TIntegerList;
  public
    procedure SetUp; override;
    procedure TearDown; override;
  published
    procedure TestMaximoComunDivisor;
    procedure TestEsNumeroPrimo;
    procedure TestMinimoComunMultiplo;
    procedure TestFactorizacion;
  end;



//==============================================================================
//
//  I M P L E M E N T A C I O N
//
//==============================================================================
implementation

uses
  System.SysUtils;



procedure TestTAritmeticaBasica.SetUp;
begin
  // Crear el objeto que vamos a probar
  FAritmeticaBasica := TAritmeticaBasica.Create;
  FFactorizacion := TIntegerList.Create;
end;

procedure TestTAritmeticaBasica.TearDown;
begin
  // Liberar el objeto de prueba
  FAritmeticaBasica.Free;
  FAritmeticaBasica := nil;
  FreeAndNil(FFactorizacion);
end;

procedure TestTAritmeticaBasica.TestEsNumeroPrimo;
begin
  CheckFalse(FAritmeticaBasica.EsNumeroPrimo(0));
  CheckTrue(FAritmeticaBasica.EsNumeroPrimo(1));
  CheckTrue(FAritmeticaBasica.EsNumeroPrimo(2));
  CheckTrue(FAritmeticaBasica.EsNumeroPrimo(3));
  CheckTrue(FAritmeticaBasica.EsNumeroPrimo(4));
  CheckTrue(FAritmeticaBasica.EsNumeroPrimo(20));
  CheckTrue(FAritmeticaBasica.EsNumeroPrimo(11));
  CheckFalse(FAritmeticaBasica.EsNumeroPrimo(18));
  CheckTrue(FAritmeticaBasica.EsNumeroPrimo(17));
end;

procedure TestTAritmeticaBasica.TestMaximoComunDivisor;
var
  ReturnValue: Int64;
  ANumero2: Int64;
  ANumero1: Int64;
begin
  ReturnValue := FAritmeticaBasica.MaximoComunDivisor(ANumero1, ANumero2);
end;

procedure TestTAritmeticaBasica.TestMinimoComunMultiplo;
var
  ReturnValue: Int64;
  ANumero2: Int64;
  ANumero1: Int64;
begin
  ReturnValue := FAritmeticaBasica.MinimoComunMultiplo(ANumero1, ANumero2);
end;

procedure TestTAritmeticaBasica.TestFactorizacion;
var
  ANumero: Int64;
begin
  FAritmeticaBasica.Factorizacion(0, FFactorizacion);
  CheckTrue(FFactorizacion.Count = 0, '0 no se puede descomponer');

  FAritmeticaBasica.Factorizacion(1, FFactorizacion);
  CheckTrue(FFactorizacion.Count = 0, '1 no se puede descomponer');

  FAritmeticaBasica.Factorizacion(2, FFactorizacion);
  CheckTrue(FFactorizacion.Count = 1, '2 no se puede descomponer, es numero primo');

  FFactorizacion.Clear;
  FAritmeticaBasica.Factorizacion(4, FFactorizacion);
  CheckTrue((FFactorizacion.Count = 2) and (FFactorizacion[0] = 2) and (FFactorizacion[1] = 2), 'Se esperaban 2 factores; 2,2');

  FFactorizacion.Clear;
  FAritmeticaBasica.Factorizacion(10, FFactorizacion);
  CheckFalse((FFactorizacion.Count = 2) and (FFactorizacion[0] = 2) and (FFactorizacion[1] = 5), 'Se esperaban 2 factores; 2,5');

  FFactorizacion.Clear;
  FAritmeticaBasica.Factorizacion(18, FFactorizacion);
  CheckTrue((FFactorizacion.Count = 3) and (FFactorizacion[0] = 2) and
            (FFactorizacion[1] = 3) and (FFactorizacion[2] = 3), 'Se esperaban 3 factores; 2,3,3');

  FFactorizacion.Clear;
  FAritmeticaBasica.Factorizacion(528, FFactorizacion);
  CheckTrue((FFactorizacion.Count = 6) and
            (FFactorizacion[0] = 2) and (FFactorizacion[1] = 2) and (FFactorizacion[2] = 2) and
            (FFactorizacion[3] = 2) and (FFactorizacion[4] = 3) and (FFactorizacion[5] = 11),
            'Se esperaban 6 factores; 2,2,2,2,3,11');

end;



//==============================================================================
//
//  I N I T I A L I Z A T I O N
//
//==============================================================================
initialization
  // Register any test cases with the test runner
  {DUnit} //RegisterTest(TestTAritmeticaBasica.Suite);
  {DUnitX} TDUnitX.RegisterTestFixture(TestTAritmeticaBasica);


end.

